#!/usr/bin/perl -w

# priced - the price daemon
#
# this daemon reads its config file
# and retrieves stocks listed in the
# file at an interval specified in the
# config file
#
# it stores this info in the file 
# /var/jstock/jstock.info
#
# this file is read by jstockd and
# served to jstock clients
#
# it expects the configuration file
# /var/jstock/priced.conf
# this file must have at the very
# least one line, beginning with
# stocks=, that lists the stocks to
# be dumped into the file.  These
# must be tickers as listed by Yahoo.com.
# Whitespace is ignored.  Ticker 
# symbols should be separated by commas.
#
# the other entry in the file should
# the variable timeout, which is the number
# of seconds between updates.

require 5.001;
use LWP::UserAgent;
use HTTP::Request;
use POSIX;

sub read_conf;
sub get_stock;
sub http_dump;
sub signal_handler;
sub phoenix;

## BEGIN MAIN ##

# GLOBAL VARIABLES ##

$time_to_die = 0;
$self = "/usr/local/bin/priced.pl";
$JSTOCK_DIR = "/var/jstock";

# /GLOBAL VARIABLES #

## DAEMONIZE ##
# When we are run, the first
# thing to do is detach from the
# terminal and become a daemon.
# Then we run in the background.
 
# change working directory
chdir($JSTOCK_DIR)
    or die "Could not chdir to /var/jstock: $!\n";

# open logfile
open (LOG, ">priced.log");
select(LOG);
$|=1;
select(STDOUT);

# redirect STDOUT and STDERR
close (STDOUT);
close (STDERR);
open (STDOUT, ">&LOG") or die "Could not redirect STDOUT: $!\n";
open (STDERR, ">&LOG") or die "Could not redirect STDERR: $!\n";

$pid = fork;
if ($pid)
{
    open (PID, ">/var/run/priced.pid");
    print PID $pid, "\n";
    close PID;
    exit;
}
die "Couldn't fork: $!" unless defined($pid);

unless (-d "/var/jstock")
{
    mkdir "/var/jstock", 0700
	or die "Could not make directory /var/jstock: $!\n";
}

 POSIX::setsid()
    or die "Can't start a new session: $!\n";

# Change the way we handle signals such
# that we die cleanly.

$SIG{INT} = $SIG{TERM} = \&signal_handler;

# Ignore sigpipe; it's not that deadly.

$SIG{PIPE} = 'IGNORE';

# Handle sighup correctly

$SIG{HUP} = \&phoenix;

# Now we begin a loop that runs as long as
# $time_to_die is 0.  Thus, we exit cleanly.

until ($time_to_die)
{
    read_conf();
    foreach $stk (@stocks)
    {
	($symbol, $price, $change) = get_stock($stk);
	if ($symbol && $price && $change)
	{
	    $symbol = ($substitutes{$symbol} || $symbol);
	    $line1 .= sprintf("%-". $spacing . "s", $symbol) . " ";
	    $line2 .= sprintf("%" . $spacing . "s", $price) . " ";
	    $line3 .= sprintf("%" . $spacing . "s", $change) . " ";
	}
	else
	{
	    print LOG "Invalid ticker symbol: $stk\n";
	}
    }
    unlink ("jstock.info");
    open (INFO, ">jstock.info");
    print  INFO $line1, "\n";
    print  INFO $line2, "\n";
    print  INFO $line3, "\n";
    close (INFO);
    sleep ($timeout);
    $line1 = $line2 = $line3 = "";
    @stocks = ();
}

close (LOG);
unlink ("/var/run/priced.pid");

## ENDOF MAIN ##

## BEGIN SUB READ_CONF ##

sub read_conf
{
    open (CONF, "priced.conf")
	or die "Couldn't open configuration file: $!\n";
    my ($time, $stocka, $stck, $rplc);
    while (<CONF>)
    {
	chomp;
        if (/timeout=(.*)$/)
	{
	    $time = $1;
	    $time =~ s/\s//g;
	    $time =~ s/[^0-9]//g;
	}
	elsif (/stocks=(.*)$/)
	{
	    $stocka = $1;
	    $stocka =~ s/\s//g;
	    push(@stocks, split(/,/, $stocka));
	}
	elsif (/spacing=(.*)$/)
	{
	    $spc = $1;
	    $spc =~ s/\s//g;
	    $spc =~ s/[^0-9]//g;
	}
	elsif (/(.*)=(.*)$/)
	{
	    $stck = $1;
	    $rplc = $2;
	    $substitutes{$stck} = $rplc;
	}
    }
    close (CONF);
    $time ||= 30;
    $timeout = $time;
    $spc ||= 18;
    $spacing = $spc;
}

## ENDOF SUB READ_CONF ##

## BEGIN SUB GET_STOCK ##

sub get_stock
{
    my $tiksym = $_[0];
    my ($symbol, $price, $change);
    my $stock = http_dump("http://finance.yahoo.com/q?s=$tiksym&d=v1");
    if ($stock =~ /<td nowrap align=left><a href=.*?>(.*?)<.*?<td.*?<td.*?<b>(.*?)<\/b>.*?<td nowrap>(.*?)<\/td>/s)
    {
	$symbol = $1;
	$price = $2;
	$change = $3;
	$price =~ s/<.*?>//g;
	$change =~ s/<.*?>//g;
	return ($symbol, $price, $change);
    }
    else
    {
	return (0, 0, 0);
    }
}

## ENDOF SUB GET_STOCK ##

## BEGIN SUB HTTP_DUMP ##

sub http_dump
{
    unless (defined $ua)
    {
	$ua = LWP::UserAgent->new();
    }
    
    return $ua->request(HTTP::Request->new(GET => $_[0]))->content;
}

## BEGIN SUB SIGNAL_HANDLER ##

sub signal_handler
{
    $time_to_die = 1;
}

## ENDOF SUB SIGNAL_HANDLER ##

## BEGIN SUB PHOENIX ##

sub phoenix
{
    exec($self) or die "Couldn't restart: $!\n";
}

## ENDOF SUB PHOENIX ##


